import 'dart:async';
import 'package:flutter/material.dart';
import 'package:nuntio_hera/hera/hera.dart';
import 'package:nuntio_hera/hera_app/helpers.dart';
import 'package:nuntio_hera/hera_app/hera_icons.dart';
import 'package:nuntio_hera/hera_app/hera_localizations.dart';
import 'package:nuntio_hera/hera_app/reigster_screen.dart';
import 'package:nuntio_hera/hera_app/send_reset_password_code_screen.dart';
import 'package:nuntio_hera/hera_app/select_registration_method_screen.dart';
import 'package:nuntio_hera/hera_app/verification_screen.dart';
import 'package:nuntio_core/models/nuntio.pb.dart';
import 'package:nuntio_core/components/iconed_button.dart';
import 'package:nuntio_core/components/nuntio_text_field.dart';
import 'package:nuntio_core/components/whatever_image.dart';
import 'package:nuntio_core/models/hera.pb.dart';
import 'package:nuntio_core/components/nuntio_state_button.dart';
import 'components/oidc_button.dart';

class LoginScreen extends StatefulWidget {
  const LoginScreen({super.key,
    required this.inputType,
    required this.config,
    required this.onAuthenticated,
  });

  final HeraConfig config;
  final TextInputType inputType;
  final Function(BuildContext context) onAuthenticated;

  @override
  State<LoginScreen> createState() => _LoginScreenState();
}

class _LoginScreenState extends State<LoginScreen> {
  final _identifierController = TextEditingController();
  final _passwordController = TextEditingController();

  bool isLoading = false;

  ButtonState _buttonState = ButtonState.idle;

  final _formKey = GlobalKey<FormState>();

  @override
  void initState() {
    super.initState();
  }

  login() {
    if (_formKey.currentState?.validate() == true) {
      setState(() {
        _buttonState = ButtonState.loading;
      });
      Hera.login(
              identifier: UserIdentifier(
                phone: validateMobile(_identifierController.text)
                    ? _identifierController.text
                    : "",
                email: validateEmail(_identifierController.text)
                    ? _identifierController.text
                    : "",
                username: validateUsername(_identifierController.text)
                    ? _identifierController.text
                    : "",
              ),
              password: _passwordController.text)
          .catchError((_) {
        setState(() {
          _buttonState = ButtonState.fail;
        });
        Timer(const Duration(seconds: 2), () {
          setState(() {
            _buttonState = ButtonState.idle;
          });
        });
      }).then((value) {
        setState(() {
          _buttonState = ButtonState.idle;
        });
        if (value.loginStatus == LoginStatus.PHONE_IS_NOT_VERIFIED) {
          Navigator.of(context)
              .push(MaterialPageRoute(
                  builder: (context) => VerificationScreen(
                        config: widget.config,
                        password: _passwordController.text,
                        loginType: LoginType.PHONE_PASSWORD,
                        expiresAt: value.expiresAt.toDateTime(),
                        phone: _identifierController.text,
                      )))
              .then((value) => widget.onAuthenticated(context));
          return;
        } else if (value.loginStatus == LoginStatus.EMAIL_IS_NOT_VERIFIED) {
          Navigator.of(context)
              .push(MaterialPageRoute(
                  builder: (context) => VerificationScreen(
                        config: widget.config,
                        password: _passwordController.text,
                        loginType: LoginType.EMAIL_PASSWORD,
                        expiresAt: value.expiresAt.toDateTime(),
                        email: _identifierController.text,
                      )))
              .then((value) => widget.onAuthenticated(context));
          return;
        }
        setState(() {
          _buttonState = ButtonState.success;
        });
        Timer(const Duration(seconds: 1), () {
          widget.onAuthenticated(context);
        });
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SingleChildScrollView(
        child: Center(
          child: Container(
            constraints: BoxConstraints(
                minHeight: MediaQuery.of(context).size.height, maxWidth: 500),
            child: SafeArea(
              child: Padding(
                padding: const EdgeInsets.only(left: 20, right: 20, bottom: 20),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  mainAxisSize: MainAxisSize.max,
                  children: [
                    WhatEverImage(widget.config.logo.url, 100),
                    Text(
                      HeraLocalizations.of(context).loginTitle,
                      style: Theme.of(context).textTheme.displayMedium,
                    ),
                    const SizedBox(height: 10),
                    Text(
                      HeraLocalizations.of(context).loginSubtitle,
                      style: Theme.of(context).textTheme.headlineSmall,
                    ),
                    const SizedBox(height: 20),
                    Column(
                      children: widget.config.oidcProviders.entries
                          .map((provider) => OidcButton(
                                context: context,
                                oidcProvider: provider.value,
                                loginSuccess: () {},
                              ))
                          .toList(),
                    ),
                    if (widget.config.oidcProviders.isNotEmpty)
                      Padding(
                        padding: const EdgeInsets.only(top: 10),
                        child: SizedBox(
                          height: 40,
                          width: double.infinity,
                          child: Row(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              const Expanded(child: Divider()),
                              const SizedBox(width: 15),
                              Text(
                                HeraLocalizations.of(context).orLoginText,
                                style: Theme.of(context)
                                    .textTheme
                                    .labelLarge
                                    ?.copyWith(
                                        color: Theme.of(context).disabledColor),
                              ),
                              const SizedBox(width: 15),
                              const Expanded(child: Divider()),
                            ],
                          ),
                        ),
                      ),
                    if (widget.config.supportedLoginMechanisms.isNotEmpty &&
                        widget.config.enableLogin)
                      Form(
                        key: _formKey,
                        child: Column(
                          children: [
                            const SizedBox(height: 10),
                            NuntioTextField(
                              prefix: HeraIcons.instance.identifier,
                              label:
                                  HeraLocalizations.of(context).identifierLabel,
                              hint:
                                  HeraLocalizations.of(context).identifierHint,
                              controller: _identifierController,
                              textInputAction: TextInputAction.next,
                              textInputType: widget.inputType,
                              validator: (text) => text?.isEmpty == true
                                  ? HeraLocalizations.of(context)
                                      .invalidIdentifierError
                                  : null,
                            ),
                            const SizedBox(height: 10),
                            NuntioTextField(
                              prefix: HeraIcons.instance.password,
                              label:
                                  HeraLocalizations.of(context).passwordLabel,
                              hint: HeraLocalizations.of(context).passwordHint,
                              controller: _passwordController,
                              textInputType: TextInputType.text,
                              textInputAction: TextInputAction.done,
                              obscureText: true,
                              validator: (text) => text?.isEmpty == true
                                  ? HeraLocalizations.of(context)
                                      .invalidPasswordError
                                  : null,
                            ),
                              Padding(
                                padding:
                                    const EdgeInsets.only(top: 5, bottom: 5),
                                child: Align(
                                  alignment: Alignment.topRight,
                                  child: TextButton(
                                    onPressed: () {
                                      Navigator.of(context).push(
                                          MaterialPageRoute(
                                              builder: (context) =>
                                                  SendResetPasswordCodeScreen(
                                                    config: widget.config,
                                                  )));
                                    },
                                    child: Text(
                                      HeraLocalizations.of(context)
                                          .forgotPasswordButton,
                                      textAlign: TextAlign.right,
                                      style: Theme.of(context)
                                          .textTheme
                                          .labelLarge,
                                    ),
                                  ),
                                ),
                              ),
                            Padding(
                              padding: const EdgeInsets.only(top: 20),
                              child: SizedBox(
                                width: double.infinity,
                                height: Theme.of(context).buttonTheme.height,
                                child: NuntioStateButton.icon(
                                  context,
                                  iconedButtons: {
                                    ButtonState.idle: IconedButton(
                                      text: HeraLocalizations.of(context)
                                          .loginButton,
                                      icon: Icon(
                                        HeraIcons.instance.login,
                                      ),
                                      color: Theme.of(context).primaryColor,
                                    ),
                                    ButtonState.loading: IconedButton(
                                      text: "",
                                      color: Theme.of(context).primaryColor,
                                    ),
                                    ButtonState.fail: IconedButton(
                                      text:
                                          HeraLocalizations.of(context).failure,
                                      icon: const Icon(Icons.cancel),
                                      color: Colors.red.shade300,
                                    ),
                                    ButtonState.success: IconedButton(
                                      text:
                                          HeraLocalizations.of(context).success,
                                      icon: const Icon(Icons.check_circle),
                                      color: Colors.green.shade400,
                                    )
                                  },
                                  state: _buttonState,
                                  onPressed: () => login(),
                                ),
                              ),
                            ),
                          ],
                        ),
                      ),
                    if (widget.config.supportedLoginMechanisms.isNotEmpty &&
                        widget.config.enableSignup)
                      Padding(
                        padding: const EdgeInsets.only(top: 10),
                        child: TextButton(
                          onPressed: () {
                            if (widget.config.supportedLoginMechanisms
                                    .contains(LoginType.USERNAME_PASSWORD) &&
                                !widget.config.supportedLoginMechanisms
                                    .contains(LoginType.EMAIL_PASSWORD) &&
                                !widget.config.supportedLoginMechanisms
                                    .contains(LoginType.PHONE_PASSWORD)) {
                              Navigator.of(context)
                                  .push(MaterialPageRoute(
                                      builder: (context) => RegisterScreen(
                                            config: widget.config,
                                            onAuthenticated:
                                                widget.onAuthenticated,
                                            loginType:
                                                LoginType.USERNAME_PASSWORD,
                                          )))
                                  .then((value) =>
                                      widget.onAuthenticated(context));
                            }
                            if (widget.config.supportedLoginMechanisms
                                    .contains(LoginType.EMAIL_PASSWORD) &&
                                !widget.config.supportedLoginMechanisms
                                    .contains(LoginType.USERNAME_PASSWORD) &&
                                !widget.config.supportedLoginMechanisms
                                    .contains(LoginType.PHONE_PASSWORD)) {
                              Navigator.of(context)
                                  .push(MaterialPageRoute(
                                      builder: (context) => RegisterScreen(
                                            config: widget.config,
                                            onAuthenticated:
                                                widget.onAuthenticated,
                                            loginType: LoginType.EMAIL_PASSWORD,
                                          )))
                                  .then((value) =>
                                      widget.onAuthenticated(context));
                            }
                            if (widget.config.supportedLoginMechanisms
                                    .contains(LoginType.PHONE_PASSWORD) &&
                                !widget.config.supportedLoginMechanisms
                                    .contains(LoginType.USERNAME_PASSWORD) &&
                                !widget.config.supportedLoginMechanisms
                                    .contains(LoginType.EMAIL_PASSWORD)) {
                              Navigator.of(context)
                                  .push(MaterialPageRoute(
                                      builder: (context) => RegisterScreen(
                                            config: widget.config,
                                            onAuthenticated:
                                                widget.onAuthenticated,
                                            loginType: LoginType.PHONE_PASSWORD,
                                          )))
                                  .then((value) =>
                                      widget.onAuthenticated(context));
                            }
                            if ((widget.config.supportedLoginMechanisms
                                        .contains(LoginType.EMAIL_PASSWORD) &&
                                    widget.config.supportedLoginMechanisms
                                        .contains(
                                            LoginType.USERNAME_PASSWORD)) ||
                                (widget.config.supportedLoginMechanisms
                                        .contains(LoginType.EMAIL_PASSWORD) &&
                                    widget.config.supportedLoginMechanisms
                                        .contains(LoginType.PHONE_PASSWORD)) ||
                                (widget.config.supportedLoginMechanisms
                                        .contains(
                                            LoginType.USERNAME_PASSWORD) &&
                                    widget.config.supportedLoginMechanisms
                                        .contains(LoginType.PHONE_PASSWORD))) {
                              Navigator.of(context)
                                  .push(MaterialPageRoute(
                                      builder: (context) =>
                                          SelectRegistrationMethodScreen(
                                            config: widget.config,
                                            onAuthenticated:
                                                widget.onAuthenticated,
                                          )))
                                  .then((value) =>
                                      widget.onAuthenticated(context));
                            }
                          },
                          child: Text(
                            HeraLocalizations.of(context).noAccountButton,
                            style: TextStyle(
                              color: Theme.of(context).primaryColor,
                            ),
                          ),
                        ),
                      )
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
