import 'package:flutter/material.dart';
import 'package:nuntio_hera/hera_app/hera_icons.dart';
import 'package:nuntio_hera/hera_app/hera_localizations.dart';
import 'package:nuntio_hera/hera_app/reigster_screen.dart';
import 'package:nuntio_core/models/hera.pb.dart';

class SelectRegistrationMethodScreen extends StatelessWidget {
  const SelectRegistrationMethodScreen({
    Key? key,
    required this.config,
    required this.onAuthenticated,
  });

  final HeraConfig config;
  final Function(BuildContext context) onAuthenticated;

  selectRegistrationMethod(BuildContext context, LoginType loginType) {
    Navigator.of(context)
        .push(MaterialPageRoute(
            builder: (context) => RegisterScreen(
                  config: config,
                  loginType: loginType,
                  onAuthenticated: onAuthenticated,
                )))
        .then((value) => Navigator.of(context).pop());
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SingleChildScrollView(
        child: Center(
          child: Container(
            height: MediaQuery.of(context).size.height,
            constraints: BoxConstraints(maxWidth: 500),
            child: SafeArea(
              child: Padding(
                padding: EdgeInsets.only(left: 20, right: 20, bottom: 20),
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    SizedBox(
                      height: 100,
                      child: Image.network(config.logo.url),
                    ),
                    Text(
                      HeraLocalizations.of(context).registerTitle,
                      style: Theme.of(context).textTheme.displayMedium,
                    ),
                    SizedBox(height: 10),
                    Text(
                      HeraLocalizations.of(context).chooseSignupMethodTitle,
                      style: Theme.of(context).textTheme.headlineSmall,
                    ),
                    SizedBox(height: 30),
                    if (config.supportedLoginMechanisms
                        .contains(LoginType.EMAIL_PASSWORD))
                      SizedBox(
                        height: Theme.of(context).buttonTheme.height,
                        child: ElevatedButton(
                          onPressed: () => selectRegistrationMethod(
                            context,
                            LoginType.EMAIL_PASSWORD,
                          ),
                          style: Theme.of(context)
                              .elevatedButtonTheme
                              .style
                              ?.copyWith(
                                backgroundColor:
                                    MaterialStateProperty.all<Color>(
                                        Theme.of(context).dividerColor),
                              ),
                          child: Row(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Icon(HeraIcons.instance.email,
                                  color: Theme.of(context).disabledColor),
                              SizedBox(width: 5),
                              Text(
                                HeraLocalizations.of(context)
                                    .registerByEmailButton,
                                textAlign: TextAlign.center,
                                style: Theme.of(context)
                                    .textTheme
                                    .bodyLarge
                                    ?.copyWith(
                                      color: Theme.of(context).disabledColor,
                                      fontWeight: FontWeight.w500,
                                      fontSize: 14,
                                    ),
                              ),
                            ],
                          ),
                        ),
                      ),
                    SizedBox(height: 8),
                    if (config.supportedLoginMechanisms
                        .contains(LoginType.PHONE_PASSWORD))
                      SizedBox(
                        height: Theme.of(context).buttonTheme.height,
                        child: ElevatedButton(
                          onPressed: () => selectRegistrationMethod(
                            context,
                            LoginType.PHONE_PASSWORD,
                          ),
                          style: Theme.of(context)
                              .elevatedButtonTheme
                              .style
                              ?.copyWith(
                                backgroundColor:
                                    MaterialStateProperty.all<Color>(
                                        Theme.of(context).dividerColor),
                              ),
                          child: Row(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Icon(HeraIcons.instance.phone,
                                  color: Theme.of(context).disabledColor),
                              SizedBox(width: 5),
                              Text(
                                HeraLocalizations.of(context)
                                    .registerByPhoneButton,
                                textAlign: TextAlign.center,
                                style: Theme.of(context)
                                    .textTheme
                                    .bodyLarge
                                    ?.copyWith(
                                      color: Theme.of(context).disabledColor,
                                      fontWeight: FontWeight.w500,
                                      fontSize: 14,
                                    ),
                              ),
                            ],
                          ),
                        ),
                      ),
                    SizedBox(height: 8),
                    if (config.supportedLoginMechanisms.contains(
                      LoginType.USERNAME_PASSWORD,
                    ))
                      SizedBox(
                        height: Theme.of(context).buttonTheme.height,
                        child: ElevatedButton(
                          onPressed: () => selectRegistrationMethod(
                            context,
                            LoginType.USERNAME_PASSWORD,
                          ),
                          style: Theme.of(context)
                              .elevatedButtonTheme
                              .style
                              ?.copyWith(
                                backgroundColor:
                                    MaterialStateProperty.all<Color>(
                                        Theme.of(context).dividerColor),
                              ),
                          child: Row(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              Icon(HeraIcons.instance.username,
                                  color: Theme.of(context).disabledColor),
                              SizedBox(width: 5),
                              Text(
                                HeraLocalizations.of(context)
                                    .registerByUsernameButton,
                                textAlign: TextAlign.center,
                                style: Theme.of(context)
                                    .textTheme
                                    .bodyLarge
                                    ?.copyWith(
                                      color: Theme.of(context).disabledColor,
                                      fontWeight: FontWeight.w500,
                                      fontSize: 14,
                                    ),
                              ),
                            ],
                          ),
                        ),
                      ),
                    SizedBox(height: 10),
                    TextButton(
                      child: Text(
                        HeraLocalizations.of(context).alreadyHaveAnAccount,
                        style: TextStyle(
                          color: Theme.of(context).primaryColor,
                        ),
                      ),
                      onPressed: () => Navigator.of(context).pop(),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }
}
